import { createAsyncThunk, createSlice } from "@reduxjs/toolkit";
import { axiosApiClient } from "../helpers/axiosApiClient";
import { FieldTypeRegister } from "@/components/Register";
import { FieldTypeSignIn } from "@/components/SignIn";

export interface IUserState {
  id: string;
  username: string;
  displayName: string;
  token: string;
}

interface State {
  user: IUserState | null;
  error: Error | null;
  loading: boolean;
}

const initialState: State = {
  user: null,
  error: null,
  loading: false
}

export const registerUser = createAsyncThunk(
    'users/register',
    async (body: FieldTypeRegister) => {
        try {
            const {data} = await axiosApiClient.post<IUserState>('/user/registration', body)
            localStorage.setItem('token', data.token)
            return data
        } catch (e) {
            throw new Error((e as Error).message)
        }
    }
)

export const signInUser = createAsyncThunk(
    'users/signInUser',
    async (body: FieldTypeSignIn) => {
        try {
            const {data} = await axiosApiClient.post<IUserState>('/user/signIn', body)
            localStorage.setItem('token', data.token)
            return data
        } catch (e) {
            throw new Error((e as Error).message)
        }
    }
)

export const validateToken = createAsyncThunk(
    'users/validateToken',
    async () => {
        try {
            const {data} = await axiosApiClient.get<IUserState>('/user/validateToken', {
                headers: {
                    Authorization: localStorage.getItem('token')
                }
            })
            console.log(data);
            
            return data
        } catch (e) {
            throw new Error((e as Error).message)
        }
    }
)

const usersSlice = createSlice(
  {
    name: 'users',
    initialState,
    reducers: {},
    extraReducers(builder) {
      builder

        // REGISTRATION
        .addCase(registerUser.fulfilled, (state, action) => {
            state.user = action.payload;
            state.loading = false;
        })
        .addCase(registerUser.rejected, (state, action) => {
            state.loading = false;
            state.error = action.error as Error;
        })
        .addCase(registerUser.pending, (state) => {
            state.loading = true;
        })

        // SIGN IN
        .addCase(signInUser.fulfilled, (state, action) => {
            state.user = action.payload;
            state.loading = false;
        })
        .addCase(signInUser.rejected, (state, action) => {
            state.loading = false;
            state.error = action.error as Error;
        })
        .addCase(signInUser.pending, (state) => {
            state.loading = true;
        })

        // VALIDATE TOKEN
        .addCase(validateToken.fulfilled, (state, action) => {
            state.user = action.payload;
            state.loading = false;
        })
        .addCase(validateToken.rejected, (state, action) => {
            state.loading = false;
            state.error = action.error as Error;
        })
        .addCase(validateToken.pending, (state) => {
            state.loading = true;
        })
    },
  }
)

export default usersSlice.reducer;
